import * as React from 'react';
import { observer } from 'mobx-react-lite';
import { checkVisible, checkDisplay, getDom, generateEmit, isSlot, CALC_RPX_ROOT_VAL_NAME, generateWidgetAPIContext, checkObserverInstanceRef, commonReport, } from '../utils';
import { get, set, isEqual } from 'lodash-es';
import { generateSlotMetaMap, getComponentChildren } from './RenderWrapper';
import { InjectContext } from '../router';
import { ErrorBoundary } from '../utils/error';
import { processCSSPropertiesUnit } from '@cloudbase/cals';
import { autorun, observable } from 'mobx';
import { REPEATER } from '../utils/constants';
import { useMemoOrDeep } from '../utils/hooks';
export const ForContext = React.createContext({});
export const ScopeContext = React.createContext({});
export const DataContext = React.createContext({});
const getSafeComponentProps = ({ style, classNameList }) => {
    const componentProps = {};
    if (classNameList.length) {
        componentProps.className = classNameList.join(' ');
    }
    if (style && Object.keys(style).length) {
        componentProps.style = style;
    }
    return componentProps;
};
// TODO: 需要不断移除 dataBinds(style/classList)
function getBindData({ widgetData, listenerInstances = [], forItems, scopeContext, codeContext, commonStyle = {}, classNameList = [], injectContext = {}, }) {
    var _a;
    // bindData
    if (Array.isArray(widgetData)) {
        widgetData =
            forItems.forIndexes === void 0 || widgetData.length === 0
                ? {}
                : get(widgetData, getForIndexes(forItems, widgetData));
    }
    widgetData = widgetData || {};
    const fieldData = Object.assign({}, widgetData);
    const isRootNode = widgetData && !widgetData.parent;
    const isInComposite = !!codeContext.$WEAPPS_COMP;
    // 再次计算 scope value
    for (let key in fieldData) {
        if (Object.prototype.hasOwnProperty.call(fieldData, key)) {
            const value = fieldData[key];
            if (value && value.__type === 'scopedValue') {
                const toBoolean = key === '_visible' || key === '_display';
                let computedValue = '';
                try {
                    computedValue = value.getValue(scopeContext || {});
                }
                catch (e) {
                    console.warn(`Error computing data bind '${key}' error:`, e);
                    computedValue = '';
                }
                fieldData[key] = toBoolean ? !!computedValue : computedValue;
            }
        }
    }
    // bindStyle
    let bindStyle = fieldData.style || {};
    /**
     * 若指令 hidden
     * 则相当于强制绑定了 display: 'none',
     */
    if (!(injectContext.checkDisplay || checkDisplay)(fieldData)) {
        bindStyle = Object.assign(Object.assign({}, bindStyle), { display: 'none' });
    }
    // 复合组件第一层需要将最外层样式 style 挂到节点上
    let cssStyle = commonStyle || {};
    let propStyle = isInComposite && isRootNode ? ((_a = codeContext.$WEAPPS_COMP.props) === null || _a === void 0 ? void 0 : _a.style) || {} : {};
    const finalStyle = getFinalStyle(injectContext.processCssUnit, cssStyle, bindStyle, propStyle);
    // bindClassList
    const bindClassList = fieldData.classList || [];
    const finalClassNameList = getFinalClassNameList([`wd-comp-id-${widgetData.id}`], classNameList, bindClassList, listenerInstances
        .filter(({ trigger }) => {
        return !trigger.includes('.');
    })
        .map(({ trigger }) => {
        return `wd-event-${trigger}`;
    }));
    // 当前在复合组件中，且当前节点为根节点
    if (isInComposite && isRootNode) {
        if (widgetData.ownerWidget) {
            Object.keys(widgetData.ownerWidget).forEach((propName) => {
                if (propName === 'role' || ['aria-', 'data-'].some((prefix) => propName.startsWith(prefix))) {
                    widgetData[propName] = widgetData.ownerWidget[propName];
                }
            });
        }
    }
    // 防止渲染时 data 的 style 与实际的 style 冲突
    delete fieldData.style;
    return {
        fieldData,
        finalStyle,
        finalClassNameList: Array.from(new Set(finalClassNameList)),
    };
}
const _FOR_ERROR_CACHE_MAP = {};
function getForList(compId, dataBinds, parentForItems, dataContext, $w, $scope) {
    /**
     * For循环渲染
     * @important
     * @default undfined // 代表没有进行for循环绑定
     */
    let forList = [];
    try {
        // 绑定了 for 变量，计算值最差应该是空数组
        // 没有绑定，那就是 undefined
        if (_FOR_ERROR_CACHE_MAP[compId]) {
            clearTimeout(_FOR_ERROR_CACHE_MAP[compId]);
        }
        let list = (dataBinds === null || dataBinds === void 0 ? void 0 : dataBinds._waFor)
            ? dataBinds._waFor(parentForItems, undefined, dataContext, $w, $scope) || []
            : undefined;
        // 绑定了for 并且计算了值，但是不是数组类型，应该进行兜底
        if (list && !Array.isArray(list)) {
            list = [];
        }
        forList = list;
    }
    catch (e) {
        if ((window === null || window === void 0 ? void 0 : window.__WEDA_LOG_LEVEL__) === 'debug') {
            // 计算值出错则使用空数组兜底
            _FOR_ERROR_CACHE_MAP[compId] = setTimeout(() => {
                _FOR_ERROR_CACHE_MAP[compId] = null;
                console.debug(`${compId}循环绑定错误`, e);
            }, 1000);
        }
        forList = [];
    }
    if (forList && !Array.isArray(forList)) {
        _FOR_ERROR_CACHE_MAP[compId] = setTimeout(() => {
            _FOR_ERROR_CACHE_MAP[compId] = null;
            console.warn(`${compId}循环绑定非数组值：`, forList);
        }, 1000);
        forList = [];
    }
    return forList;
}
// FieldWrapper 依赖 currentWidgets scope 因此需要添加observer
const FieldWrapper = observer(_FieldWrapper);
function useObserverableRef(sourceKey = '') {
    const pureRef = React.useRef();
    const ref = React.useRef();
    const observerRef = React.useMemo(() => {
        return checkObserverInstanceRef(sourceKey) ? observable(ref) : pureRef;
    }, []);
    return observerRef;
}
function useWidgetInstanceRef(widget) {
    const ref = useObserverableRef(widget === null || widget === void 0 ? void 0 : widget.widgetType);
    const _instanceRef = React.useRef();
    return React.useMemo(() => {
        if (widget) {
            if (widget._instanceRef !== _instanceRef.current) {
                _instanceRef.current = widget._instanceRef;
                widget._instanceRef.current = null;
            }
            return widget._instanceRef;
        }
        return ref;
    }, [widget === null || widget === void 0 ? void 0 : widget._instanceRef, ref]);
}
const emptyObject = {};
function _FieldWrapper(props) {
    var _a, _b, _c, _d, _e;
    const { Field, componentSchema, id, data, style: _style = emptyObject, className = '', events: _events, compositeParent, context, updateContext, renderSlot = false, } = props;
    const style = useMemoOrDeep(_style, false);
    const events = useMemoOrDeep(_events, false);
    const contextRef = React.useRef(context);
    contextRef.current = context;
    const { forContext, forIndexes, widgetsData, codeContext, 
    //  dataContext,
    virtualFields, } = context;
    // if (['container116', 'container2'].includes(id)) {
    //   console.log('>>>>>> FieldWrapper', id);
    //   trace(true);
    // }
    const injectContext = React.useContext(InjectContext);
    const scopeContext = React.useContext(ScopeContext);
    const _dataContext = React.useContext(DataContext);
    const indexRef = React.useRef();
    const typeRef = React.useRef();
    const [_, forceUpate] = React.useState(0);
    const isInComposite = !!codeContext.$WEAPPS_COMP;
    const isDataContainer = (_b = (_a = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.extra) === null || _a === void 0 ? void 0 : _a.compConfig) === null || _b === void 0 ? void 0 : _b.isDataContainer;
    const currentWidget = Array.isArray(widgetsData) ? get(widgetsData, forIndexes) : widgetsData;
    const instanceRef = useWidgetInstanceRef(currentWidget);
    // context 变化强刷一次children，同时把 currentWidget._scope?.dataContext  添加到 observer 中
    const dispose = React.useMemo(() => {
        return autorun((r) => {
            var _a;
            // r.trace();
            if (isDataContainer && ((_a = currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope) === null || _a === void 0 ? void 0 : _a.dataContext) && Object.values(currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope.dataContext)) {
                // if (['container116', 'container2'].includes(id)) {
                //   console.log('***');
                // }
                forceUpate((_) => {
                    return _ + 1;
                });
            }
        });
    }, []);
    React.useEffect(() => {
        return () => {
            dispose();
        };
    }, []);
    const totalContext = React.useMemo(() => {
        var _a;
        const res = isDataContainer && ((_a = currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope) === null || _a === void 0 ? void 0 : _a.dataContext)
            ? Object.assign(Object.assign({}, _dataContext), { [id]: currentWidget._scope.dataContext }) : _dataContext;
        return res;
    }, [_dataContext, _]);
    const watchedList = isInComposite
        ? [componentSchema]
        : [componentSchema, scopeContext, codeContext, injectContext, virtualFields];
    let staticResourceAttribute = ((_c = componentSchema.extra) === null || _c === void 0 ? void 0 : _c.staticResourceAttribute) || [];
    // 组件最终用于执行的事件函数
    const emit = React.useCallback(generateEmit((_e = (_d = (!isInComposite ? codeContext.$page : codeContext.$WEAPPS_COMP)) === null || _d === void 0 ? void 0 : _d.__internal__) === null || _e === void 0 ? void 0 : _e.$w, currentWidget, injectContext, isDataContainer
        ? [
            {
                key: `wa${Date.now().toString().slice(-8)}`,
                trigger: 'onDataChange',
                isCapturePhase: false,
                data: {},
                dataBinds: {},
                instanceFunction: ({ event }) => {
                    var _a, _b, _c, _d, _e, _f;
                    if (currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._scope) {
                        if (currentWidget._scope.dataContext) {
                            if (!isEqual(currentWidget._scope.dataContext.data, (_a = event === null || event === void 0 ? void 0 : event.detail) === null || _a === void 0 ? void 0 : _a.data)) {
                                currentWidget._scope.dataContext.data = (_b = event === null || event === void 0 ? void 0 : event.detail) === null || _b === void 0 ? void 0 : _b.data;
                            }
                            if (!isEqual(currentWidget._scope.dataContext.state, (_c = event === null || event === void 0 ? void 0 : event.detail) === null || _c === void 0 ? void 0 : _c.state)) {
                                currentWidget._scope.dataContext.state = (_d = event === null || event === void 0 ? void 0 : event.detail) === null || _d === void 0 ? void 0 : _d.state;
                            }
                        }
                        else {
                            currentWidget._scope.dataContext = { data: (_e = event === null || event === void 0 ? void 0 : event.detail) === null || _e === void 0 ? void 0 : _e.data, state: (_f = event === null || event === void 0 ? void 0 : event.detail) === null || _f === void 0 ? void 0 : _f.state };
                        }
                    }
                    return event.detial;
                },
            },
        ]
        : [], {}), [currentWidget]);
    /**
     * 组件最终传给组件props的，要保持引用防rerender
     * useComponentProps 保持只用单一引用
     * 通过 ref 获取最新的 context 值
     */
    const emitFn = React.useCallback((trigger, eventData, originalEvent) => {
        var _a;
        return emit(trigger, componentSchema.listenerInstances, eventData, contextRef.current.forContext, originalEvent, scopeContext, totalContext, !isInComposite ? (_a = codeContext.$page) === null || _a === void 0 ? void 0 : _a.id : '');
    }, [componentSchema.listenerInstances, contextRef, totalContext]);
    React.useLayoutEffect(() => {
        return () => {
            instanceRef.current = null;
        };
    }, []);
    React.useLayoutEffect(() => {
        return () => {
            instanceRef.current = null;
        };
    }, [currentWidget]);
    if (!Array.isArray(staticResourceAttribute)) {
        staticResourceAttribute = [];
    }
    // 判断为容器组件，则增加一个onDataChange事件
    if (isDataContainer && !(events === null || events === void 0 ? void 0 : events.includes('onDataChange'))) {
        events.push('onDataChange');
    }
    // 多个组件的 slot 属性
    const slotMetaMap = React.useMemo(() => {
        return generateSlotMetaMap(componentSchema, {
            scopeContext,
            codeContext,
            virtualFields,
            updateContext,
            injectContext,
        }, {
            renderSlot,
        });
    }, watchedList);
    const slotMap = React.useMemo(() => {
        return generateSlotMap(slotMetaMap, {
            forContext,
            dataContext: totalContext,
        });
    }, [slotMetaMap, totalContext, forContext]);
    const _selectableBlockEvents = React.useMemo(() => {
        const _selectableBlockEmit = (trigger, eventData, originalEvent) => {
            var _a, _b, _c;
            return emit(trigger, typeRef.current &&
                indexRef.current !== undefined &&
                ((_c = (_b = (_a = data === null || data === void 0 ? void 0 : data[typeRef.current]) === null || _a === void 0 ? void 0 : _a[indexRef.current]) === null || _b === void 0 ? void 0 : _b.selectableBlock) === null || _c === void 0 ? void 0 : _c.listenerInstances), eventData, contextRef.current.forContext, originalEvent, scopeContext, totalContext);
        };
        const r = {
            // 选区的预览的click事件
            onCustomEvent: ({ order: index, blockKey }) => {
                if (index !== undefined) {
                    indexRef.current = index;
                    typeRef.current = blockKey;
                }
            },
            events: new Proxy({}, {
                get(obj, prop) {
                    return prop in obj ? obj[prop] : (e) => _selectableBlockEmit(prop, e);
                },
            }),
            emit: _selectableBlockEmit,
        };
        return r;
    }, [componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.selectableBlock, totalContext, scopeContext, contextRef]);
    // React.useEffect(() => {
    //   if (id === 'repeater120' || id === 'repeater1') {
    //     console.log('componentSchema?.selectableBlock, totalContext', componentSchema?.selectableBlock, totalContext);
    //   }
    // }, [componentSchema?.selectableBlock, totalContext]);
    // React.useEffect(() => {
    //   if (id === 'repeater120' || id === 'repeater1') {
    //     console.log(' scopeContext, contextRef', scopeContext, contextRef);
    //   }
    // }, [scopeContext, contextRef]);
    return (React.createElement(ErrorBoundary, { FallbackComponent: ({ error }) => {
            console.error(`组件${id}发生了错误，：${error}`);
            return null;
        }, onError: (error, info) => {
            var _a;
            commonReport({ message: `${((_a = codeContext === null || codeContext === void 0 ? void 0 : codeContext.$page) === null || _a === void 0 ? void 0 : _a.id) || ''} 组件${id}发生了错误，${error.message}`, level: 4 });
        } },
        React.createElement(DataContext.Provider, { value: totalContext },
            React.createElement(Field, { ref: instanceRef, data: Object.assign(Object.assign(Object.assign({}, data), slotMap), { _selectableBlockEvents }), id: id, emit: emitFn, events: events, 
                /**
                 * @deprecated
                 * 老的复合组件
                 * 和 Slot 组件依赖
                 * 实际上应废弃
                 */
                compositeParent: compositeParent, forIndexes: forIndexes, "$node": currentWidget, "$widget": currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget._userWidget, domRef: setGetDomApi(currentWidget, { codeContext }), style: style, className: className, staticResourceAttribute: staticResourceAttribute, isInComposite: isInComposite }, getComponentChildren(componentSchema, {
                virtualFields,
                codeContext,
                forContext,
                injectContext,
                scopeContext,
                dataContext: totalContext,
                updateContext,
            })))));
}
function SlotElementWrapper({ component, forContext, dataContext, codeContext = undefined }) {
    return component({ forContext, dataContext, codeContext });
}
function generateSlotMap(slots, { forContext, dataContext, codeContext = undefined }) {
    let map = {};
    Object.keys(slots).forEach((slotProp) => {
        let meta = slots[slotProp];
        if (meta.type === 'HOC') {
            set(map, slotProp, meta.node({ forContext, dataContext }));
        }
        else {
            set(map, slotProp, React.createElement(SlotElementWrapper, { component: meta.node, forContext: forContext, dataContext: dataContext, codeContext: codeContext }));
        }
    });
    return map;
}
function SlotWrapper(props) {
    return props.children;
}
function Page(props) {
    return props.children;
}
export function getComponentRenderList(props) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    const { key, virtualFields, renderSlot, codeContext = {}, scopeContext = {}, genericComp = { propName: '' }, emitEvents = [], componentSchema = {}, forContext = {}, injectContext = {}, rootNode = false, dataContext, updateContext, } = props;
    const compId = componentSchema.id;
    const isInComposite = !!codeContext.$WEAPPS_COMP;
    // 判断 widgets 是从 page 来的，还是组件来的
    const widgetsData = !isInComposite ? codeContext.$page.widgets[compId] : codeContext.$WEAPPS_COMP.widgets[compId];
    const classNameList = (((_a = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.attributes) === null || _a === void 0 ? void 0 : _a.class) || '').split(/\s/).filter((item) => !!item);
    const dataBinds = ((_b = codeContext._dataBinds) === null || _b === void 0 ? void 0 : _b[compId]) || {};
    const FieldKey = genericComp.propName
        ? codeContext.node[genericComp.propName]
        : `${componentSchema.module}:${componentSchema.component}`;
    const Field = componentSchema.type === 'PAGE' ? Page : virtualFields[FieldKey];
    const context = {
        scopeContext,
        forContext,
        codeContext,
        dataContext,
        virtualFields,
    };
    if (isSlot(componentSchema)) {
        const SlotRender = !compId ? SlotWrapper : injectContext.SlotRender || SlotWrapper;
        const slotId = `${(_d = (_c = componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.extra) === null || _c === void 0 ? void 0 : _c.__parent) === null || _d === void 0 ? void 0 : _d.id}.${compId}`;
        return (React.createElement(SlotRender, { component: slotId, componentSchema: componentSchema, isComposite: isInComposite }, getComponentChildren(componentSchema, Object.assign(Object.assign({}, context), { injectContext,
            updateContext }))));
    }
    /**
     * 这里逻辑的增加是为了支持 Repeater 组件和 $w 作用域
     * 此逻辑作用于 2 层循环组件绑定了 1 层循环项的数组值
     * 1. 补全 forContext.forStack
     * 2. 为 getForList 的 dataBinds._waFor 方法提供正确的 widget 实例
     */
    let currentForParentWidget = Array.isArray(widgetsData) ? get(widgetsData, forContext.forIndexes || []) : widgetsData;
    /**
     * get forIndexes 后仍为数组则说明当前节点是绑定for循环的
     * 会再多一级，此刻仅为了验证
     */
    if (Array.isArray(currentForParentWidget)) {
        /**
         * 提前判断纯属是为了减少 mobx warn
         * 不然访问空数组的[0]总是打一个 warn 出来
         */
        if (currentForParentWidget.length) {
            currentForParentWidget = (_e = currentForParentWidget[0]) === null || _e === void 0 ? void 0 : _e.parent;
        }
        else {
            currentForParentWidget = undefined;
        }
    }
    // For循环渲染
    let forList = getForList(compId, dataBinds, forContext, dataContext, generateWidgetAPIContext((_g = (_f = (!isInComposite ? codeContext.$page : codeContext.$WEAPPS_COMP)) === null || _f === void 0 ? void 0 : _f.__internal__) === null || _g === void 0 ? void 0 : _g.$w, currentForParentWidget, forContext, scopeContext), scopeContext);
    if (forList) {
        return forList
            .map((item, index) => {
            var _a, _b, _c;
            let { forIndexes = [], forStack = [] } = forContext;
            const forItemsIndexes = forIndexes.concat(index);
            const currentWidget = Array.isArray(widgetsData) ? get(widgetsData, forItemsIndexes) : widgetsData;
            let forContextListAlias;
            const listMeta = { currentItem: item, currentIndex: index };
            switch (currentWidget === null || currentWidget === void 0 ? void 0 : currentWidget.widgetType) {
                case `${REPEATER.MODULE_NAME}:${REPEATER.REPEATER_ITEM_NAME}`:
                case `${REPEATER.DEBUG_MODULE_NAME}:${REPEATER.REPEATER_ITEM_NAME}`: {
                    const repeaterWidget = currentWidget.parent;
                    forContextListAlias = {
                        [`${repeaterWidget.forIndex}` || 'currentIndex']: listMeta.currentIndex,
                        [`${repeaterWidget.forItem}` || 'currentItem']: listMeta.currentItem,
                    };
                }
            }
            const forItems = Object.assign(Object.assign({}, forContext), { [compId]: item, forIndexes: forItemsIndexes, forStack: [
                    ...forStack,
                    Object.assign(Object.assign({}, listMeta), { alias: forContextListAlias }),
                ] });
            const { fieldData: forItemData, finalStyle: forItemStyle, finalClassNameList: forItemClassNameList, } = getBindData({
                widgetData: widgetsData,
                listenerInstances: componentSchema.listenerInstances,
                forItems,
                scopeContext,
                codeContext,
                commonStyle: (_a = componentSchema.extra) === null || _a === void 0 ? void 0 : _a.commonStyle,
                classNameList,
                injectContext: Object.assign(Object.assign({}, injectContext), { processCssUnit: isInComposite
                        ? ((_c = (_b = codeContext.$WEAPPS_COMP) === null || _b === void 0 ? void 0 : _b.__internal__) === null || _c === void 0 ? void 0 : _c.defaultCssUnit) || 'rpx'
                        : injectContext.processCssUnit }),
            });
            const { VisiblePlaceHolder } = injectContext;
            if (!(injectContext.checkVisible || checkVisible)(forItemData)) {
                return VisiblePlaceHolder ? React.createElement(VisiblePlaceHolder, { id: compId }) : null;
            }
            const filedElement = (React.createElement(ForContext.Provider, { key: forItemData._key, value: forItems },
                React.createElement(FieldWrapper, Object.assign({ Field: Field, componentSchema: componentSchema, context: Object.assign(Object.assign({}, context), { widgetsData, forContext: forItems, forIndexes: forItemsIndexes }), id: compId, updateContext: updateContext, data: forItemData }, getSafeComponentProps({
                    style: forItemStyle,
                    classNameList: forItemClassNameList,
                }), { events: emitEvents, compositeParent: codeContext, renderSlot: renderSlot }))));
            return !(injectContext.checkDisplay || checkDisplay)(forItemData) && VisiblePlaceHolder ? (React.createElement(VisiblePlaceHolder, { id: compId }, filedElement)) : (filedElement);
        })
            .filter((item) => !!item);
    }
    // 修正 forIndexes
    const forIndexes = getForIndexes(forContext, widgetsData);
    // 单节点渲染
    const { fieldData, finalClassNameList, finalStyle } = getBindData({
        widgetData: widgetsData,
        listenerInstances: componentSchema.listenerInstances,
        forItems: forContext,
        scopeContext,
        codeContext,
        commonStyle: (_h = componentSchema.extra) === null || _h === void 0 ? void 0 : _h.commonStyle,
        classNameList,
        injectContext: Object.assign(Object.assign({}, injectContext), { processCssUnit: isInComposite
                ? ((_k = (_j = codeContext.$WEAPPS_COMP) === null || _j === void 0 ? void 0 : _j.__internal__) === null || _k === void 0 ? void 0 : _k.defaultCssUnit) || 'rpx'
                : injectContext.processCssUnit }),
    });
    // false 或空字符串时
    if (!(injectContext.checkVisible || checkVisible)(fieldData)) {
        const { VisiblePlaceHolder } = injectContext;
        return VisiblePlaceHolder ? [React.createElement(VisiblePlaceHolder, { key: key, id: compId })] : [];
    }
    return [
        React.createElement(FieldWrapper, Object.assign({ key: key, Field: Field, componentSchema: componentSchema, context: Object.assign(Object.assign({}, context), { widgetsData,
                forIndexes }), id: compId, updateContext: updateContext, data: fieldData }, getSafeComponentProps({
            style: finalStyle,
            classNameList: finalClassNameList,
        }), { events: emitEvents, compositeParent: codeContext, renderSlot: renderSlot })),
    ];
}
export const CompRenderer = observer(function (props) {
    return getComponentRenderList(Object.assign(Object.assign({}, props), { scopeContext: React.createContext(ScopeContext), forContext: React.useContext(ForContext), injectContext: React.useContext(InjectContext) }));
});
function getFinalStyle(type, commonStyle = {}, bindStyle = {}, propStyle = {}) {
    const processOptions = {
        looseError: true,
        rpxCalcRoot: `var(${CALC_RPX_ROOT_VAL_NAME})`,
    };
    const processCssUnit = type || 'rem';
    const remBindStyle = typeof bindStyle === 'object' ? processCSSPropertiesUnit(bindStyle, processCssUnit, processOptions) : {};
    return Object.assign(Object.assign(Object.assign({}, (processCSSPropertiesUnit(commonStyle, processCssUnit, processOptions) || {})), remBindStyle), propStyle);
}
function getFinalClassNameList(...args) {
    return args.reduce((list, item = []) => {
        if (Array.isArray(item)) {
            list.push(...item);
        }
        else {
            list.push(item);
        }
        return list;
    }, []);
}
// HACK: 从后向前保证循环的深度与 forIndexes 一致
// 后续需要将 For 循环迁移到外层
export function getForIndexes(parentForItems, widgetsData) {
    return Array.isArray(widgetsData) && widgetsData.length > 0
        ? (parentForItems.forIndexes || []).slice(0 - getDeepArrLen(widgetsData))
        : undefined;
}
function getDeepArrLen(arr, len = 0) {
    if (Array.isArray(arr)) {
        return getDeepArrLen(arr.length > 0 ? arr[0] : null, len + 1);
    }
    else {
        return len;
    }
}
function setGetDomApi(currentWidget, { codeContext }) {
    if (!currentWidget)
        return React.createRef();
    const isComposite = !currentWidget.widgetType.startsWith('gsd-h5-react');
    const isInComposite = !!(codeContext === null || codeContext === void 0 ? void 0 : codeContext.$WEAPPS_COMP);
    // 如果当前是复合组件，不做 getDom 的挂载
    if (!isComposite) {
        if (!currentWidget.domRef) {
            currentWidget.domRef = React.createRef();
        }
        if (!currentWidget.getDom) {
            currentWidget.getDom = (options) => getDom(currentWidget.domRef.current, options);
        }
        if (isInComposite && // 当前在复合组件内
            !currentWidget.parent && // 当前节点为复合组件的根节点
            codeContext.node && // 复合组件的 node 已经创建
            !codeContext.node.getDom // 复合组件的 node 未挂载 getDom 方法
        ) {
            codeContext.node.domRef = currentWidget.domRef;
            codeContext.node.getDom = currentWidget.getDom;
        }
    }
    return currentWidget.domRef;
}
